#FastGWR MPI Script
#Author: Ziqi Li
#Email: liziqi1992@gmail.com

import argparse
import numpy as np
from mpi4py import MPI
from FastGWR import FastGWR
from FastMGWR import FastMGWR

#Direct Example Call:
#You can direct call this script by:
#mpiexec -np 4 python fastgwr_mpi.py -data ../Zillow-test-dataset/zillow_1k.csv
#mpiexec -np 4 python fastgwr_mpi.py -data ../Zillow-test-dataset/zillow_1k.csv -mgwr -c

if __name__ == "__main__":

    #Initializing MPI
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    size = comm.Get_size()
    
    #Parsing arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("-data")
    parser.add_argument("-out",default="fastgwr_rslt.csv")
    parser.add_argument("-bw")
    parser.add_argument("-minbw")
    parser.add_argument("-chunks",default=1)
    
    parser.add_argument('-estonly',action='store_true')
    parser.add_argument('-mgwr',action='store_true')
    parser.add_argument('-f','--fixed',action='store_true')
    parser.add_argument('-a','--adaptive',action='store_true')
    parser.add_argument('-c','--constant',action='store_true')

    
    #Timing starts
    t1 = MPI.Wtime()
    n_chunks = parser.parse_args().chunks
    
    #Fitting MGWR model, if mgwr in the arguments
    if parser.parse_args().mgwr:
            mgwr_model = FastMGWR(comm,parser)
            mgwr_model.backfitting()
            mgwr_model.mgwr_fit(int(n_chunks))
    
    #Fitting GWR model
    else:
        FastGWR(comm,parser).fit()
    #Timing ends
    t_last = MPI.Wtime()
    
    wt = comm.gather(t_last-t1, root=0)
    if rank ==0:
        print("Total Time Elapsed:",np.round(max(wt),2),"seconds")
        print("-"*60)






from setuptools import setup, find_packages
from io import open
from os import path

import pathlib
# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# automatically captured required modules for install_requires in requirements.txt
with open(path.join(HERE, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if ('git+' not in x) and (
    not x.startswith('#')) and (not x.startswith('-'))]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs \
                    if 'git+' not in x]


setup(
    name="fastgwr",
    version='0.2.8',
    description="Fast Parallel Computation of Geographically Weighted Regression",
    long_description="A MPI-based command line tool for calibrating Geographically Weighted Regression models.",
    author="Ziqi Li",
    author_email="liziqi1992@gmail.com",
    url="https://github.com/Ziqi-Li/FastGWR",
    license="MIT",
    entry_points={'console_scripts': ['fastgwr=fastgwr.__main__:main',],},
    dependency_links=dependency_links,
    install_requires = install_requires,
    packages = find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ]
)

